<?php
namespace SupplyTek\Services;

/**
 * 
 * Connects to the database to perform queries related to the order
 *  
 * @author Grazielle Ferreira Antunes
 */
class OrdersService
{
    protected $connection;
    protected $newCodeSC;

    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param object $entityManager Object to manage database connection and queries
     * 
     * @return void
     * 
     * @access public
     * @author Grazielle Ferreira Antunes
     */
    public function __construct($entityManager, $newCodeSC)
    {
        $this->connection = $entityManager->getConnection();
        $this->newCodeSC = $newCodeSC;
    }
    
    /**
     * Function that begin database transaction
     * 
     * @return void
     * 
     * @access public
     * @author Grazielle Ferreira Antunes
     */
    public function beginTransaction () {
        $this->connection->beginTransaction();
    }
    
    /**
     * Function that commit database modifications and close transaction
     * 
     * @return void
     * 
     * @access public
     * @author Grazielle Ferreira Antunes
     */
    public function commit () {
        $this->connection->commit();
    }
    
    /**
     * Function that rollback database modifications and close transaction
     * 
     * @return void
     * 
     * @access public
     * @author Grazielle Ferreira Antunes
     */
    public function rollback () {
        $this->connection->rollback();
    }

    
    /**
     * Insert comment regarding the order with or without attachment
     * 
     * @param array $formData – User-submitted comment
     * @param array $order – Order information
     * @param array $fileRow – Information about the attached file
     * @param array $sessionData – Data about the logged-in session (CDOPERADOR and NRORG)
     *
     * @access public
     * 
     * @author Grazielle Ferreira Antunes
     */
    public function insertOrderComment($formData, $order, $fileRow, $sessionData) 
    {
        $newCode = $this->newCodeSC->getCodeByTableName('ANEXOLANCAF', $sessionData['NRORG']);

        $sql = "INSERT INTO ANEXOLANCAF (
                    NRANEXOLANCAF,
                    NRAUTOFORN,
                    CDTELA,
                    CDFILDEST,
                    DTANEXOLANCAF,
                    NMANEXOLANCAF,
                    NMPATHANEXO,
                    DSANEXOLANCAF,
                    DSINFOANEXO,
                    CDOPERINCLUSAO,
                    DTINCLUSAO,
                    NRORGINCLUSAO,
                    NRORG
                ) VALUES (
                    :NRANEXOLANCAF,
                    :NRAUTOFORN,
                    :CDTELA,
                    :CDFILDEST,
                    SYSDATE,
                    :NMANEXOLANCAF,
                    :NMPATHANEXO,
                    :DSANEXOLANCAF,
                    :DSINFOANEXO,
                    :CDOPERINCLUSAO,
                    SYSDATE,
                    :NRORG,
                    :NRORG
                )";
                
        $params = array(
            "NRANEXOLANCAF"  => $newCode,
            "NRAUTOFORN"     => $order['ID'],
            "CDTELA"         => 'SUPPLY',
            "CDFILDEST"      => $order['BRANCH'],
            "NMANEXOLANCAF"  => 'Comentario SupplyTek',
            "NMPATHANEXO"    => $fileRow['path'],
            "DSANEXOLANCAF"  => $fileRow['name'],
            "DSINFOANEXO"    => $formData['commentText'],
            "CDOPERINCLUSAO" => $sessionData['CDOPERADOR'],
            "NRORG"          => $sessionData['NRORG']
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }
    
    /**
     * Updates the product quantity in the order as modified by the user
     *
     * @param array $row – Order information and modified quantity
     * @param string $nrOrg – Logged-in organization number
     *
     * @access public
     * 
     * @author Grazielle Ferreira Antunes
     */
    public function updateItemsFromOrder($row, $nrOrg) 
    {
        $sql = "UPDATE ITEMAUTF
                SET
                    QTPROGENTAF =:QTPROGENTAF
                WHERE
                    NRAUTOFORN= :NRAUTOFORN
                    AND CDPRODUTO = :CDPRODUTO
                    AND CDFILDEST = :CDFILDEST
                    AND CDFILLANC = :CDFILLANC
                    AND CDMARCA = :CDMARCA
                    AND DTPROGENTAF = :DTPROGENTAF
                    AND NRORG = :NRORG";
                
        $params = array(
            "QTPROGENTAF" => $row['QUANTITY'],
            "NRAUTOFORN"  => $row['ORDER_ID'],
            "CDPRODUTO"   => $row['PRODUCT_ID'],
            "CDFILDEST"   => $row['CDFILDEST'],
            "CDFILLANC"   => $row['CDFILLANC'],
            "CDMARCA"     => $row['CDMARCA'],
            "DTPROGENTAF" => $row['DTPROGENTAF'],
            "NRORG"       => $nrOrg
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
    }
    
    public function validateDeliveryDate ($unitId, $supplierId, $deliveryDate)
    {
        $sql = "SELECT CARREGA_PERIFORN_FUN(:P_CDFILIAL, :P_CDFORNECED, :P_DTENTREGA) DTENTREGA FROM DUAL";
        
        $params = array(
            "P_CDFILIAL"   => $unitId,
            "P_CDFORNECED" => $supplierId,
            "P_DTENTREGA"  => $deliveryDate
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
        $result = $std->fetch();
        return isset($result["DTENTREGA"]) ? $result["DTENTREGA"] : null;
    }
    
    public function getDeliveryDate ($unitId, $supplierId)
    {
        $sql = "SELECT CDPERIENTR FROM REFORFIL WHERE CDFORNECED = :P_CDFORNECED AND CDFILIAL = :P_CDFILIAL";
        
        $params = array(
            "P_CDFORNECED" => $supplierId,
            "P_CDFILIAL"   => $unitId
        );

        $std = $this->connection->prepare($sql);
        
        $std->execute($params);
        $result = $std->fetch();
        $cdperientr = isset($result["CDPERIENTR"]) ? $result["CDPERIENTR"] : null;
        
        $sql2 = "SELECT DTENTREGA FROM CALECOMP WHERE CDFILIAL = :P_CDFILIAL AND CDPERIENTR = :P_CDPERIENTR AND DTENTREGA >= SYSDATE and ROWNUM <= 1";
        
        $params2 = array(
            "P_CDFILIAL"   => $unitId,
            "P_CDPERIENTR" => $cdperientr
        );

        $std2 = $this->connection->prepare($sql2);
        
        $std2->execute($params2);
        $result2 = $std2->fetch();
        return isset($result2["DTENTREGA"]) ? $result2["DTENTREGA"] : null;
    }

}